      subroutine readHB(lunit,m,n)
      implicit none
c
c     Read matrix in Harwell-Boeing format
c
      include 'HB.h'
       
      integer lunit,m,n
      CHARACTER      TITLE*72 , KEY*8    , MXTYPE*3 , RHSTYP*3,
     1               PTRFMT*16, INDFMT*16, VALFMT*20, RHSFMT*20

      INTEGER        TOTCRD, PTRCRD, INDCRD, VALCRD, RHSCRD,
     1               NELTVL, I,  NRHS  , NRHSIX
      logical lsame
      external lsame

C    ------------------------
C     ... READ IN HEADER BLOCK
C     ------------------------

      read ( lunit, 1000 ) title , key   ,
     1                     totcrd, ptrcrd, indcrd, valcrd, rhscrd,
     2                     mxtype, nrow  , ncol  , nnzero, neltvl,
     3                     ptrfmt, indfmt, valfmt, rhsfmt
      if  ( rhscrd .gt. 0 )
     1    read ( lunit, 1001 ) rhstyp, nrhs, nrhsix
 1000 format ( a72, a8 / 5i14 / a3, 11x, 4i14 / 2a16, 2a20 )
 1001 format ( a3, 11x, 2i14 )

      write (*,1000) title , key   ,
     1                     totcrd, ptrcrd, indcrd, valcrd, rhscrd,
     2                     mxtype, nrow  , ncol  , nnzero, neltvl,
     3                     ptrfmt, indfmt, valfmt, rhsfmt
c     -------------------------
C     ... READ MATRIX STRUCTURE
C     -------------------------

      if (ncol.gt.nmax) stop 'ERROR in readHB: ncol > nmax'
      if (nrow.gt.mmax) stop 'ERROR in readHB: nrow > mmax'
      if (nnzero.gt.nnzmax) stop 'ERROR in readHB: nnzero > nnzmax'
      

      read ( lunit, ptrfmt ) ( colptr (i), i = 1, ncol+1 )
      read ( lunit, indfmt ) ( rowind (i), i = 1, nnzero )

      if  ( valcrd .gt. 0 )  then

c         ----------------------
c         ... read matrix values
c         ----------------------

          read ( lunit, valfmt ) ( values (i), i = 1, nnzero )

      endif

      if (lsame(mxtype(2:2),'U').or.lsame(mxtype(2:2),'R')) then
         issym = 0
      else if (lsame(mxtype(2:2),'S')) then
         issym = 1
      else
         print *, 'unsupported format',mxtype
         stop 
      endif

      m = nrow
      n = ncol
      end


      subroutine hbmv(transa,m,n,x,y,dparm,iparm)
      implicit none
      character*1 transa
      integer m,n,iparm(*)
      double precision x(*),y(*),dparm(*)
      logical lsame
      external lsame
      
      if (lsame(transa,'n')) then
         call avHB (m, n, x, y)
      else
         call atvHB (m, n, x, y)
      endif
      end

      subroutine avHB (m, n, x, y)
c
c     computes  y <- A*x
c
      implicit none
      integer          m, n, i, j
      Double precision x(n), y(m)
      include 'HB.h'
      

      do i=1,m
         y(i) = 0d0
      enddo
      if (issym.eq.1) then
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
               y(rowind(j)) = y(rowind(j)) + values(j)*x(rowind(j))
            enddo
         enddo
      else
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(rowind(j)) = y(rowind(j)) + values(j)*x(i)
            enddo
         enddo
      endif
      end

      subroutine atvHB (m, n, x, y)
c
c     computes  y <- A'*x
c     
      integer          m, n, i, j
      Double precision x(m), y(n)
      include 'HB.h'
      
      do i=1,n
         y(i) = 0d0
      enddo
      if (issym.eq.1) then
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
               y(rowind(j)) = y(rowind(j)) + values(j)*x(rowind(j))
            enddo
         enddo
      else
         do i=1,n
            do j=colptr(i),colptr(i+1)-1
               y(i) = y(i) + values(j)*x(rowind(j))
            enddo
         enddo
      endif
      end




